BASE_URL = "base_url"
USERNAME = "username"
PASSWORD = "password"
PROJECT_GROUP_NAME = "project_group_name"
PROJECT_NAME = "project_name"
PROJECT_VERSION = "project_version"
DEFAULT_VERSION = "default"

# API HTTP Request Methods
API_DELETE = "delete"
API_GET = "get"
API_POST = "post"
API_PUT = "put"
